import { Color, } from 'three';
import { SceneManager } from '../../three-manager/scene-manager';
import { ColorByOptionKeys, ColorOptions } from '../color-options';
/**
 * A wrapper class for Phoenix menu to perform UI related operations.
 */
export class PhoenixMenuUI {
    /**
     * Create Phoenix menu UI with different controls related to detector geometry and event data.
     * @param phoenixMenuRoot Root node of the Phoenix menu.
     * @param three The three manager for managing three.js related operations.
     */
    constructor(phoenixMenuRoot, three) {
        this.phoenixMenuRoot = phoenixMenuRoot;
        this.three = three;
        this.sceneManager = three.getSceneManager();
    }
    /**
     * Clear the menu by removing all folders.
     */
    clear() {
        if (this.phoenixMenuRoot) {
            this.phoenixMenuRoot.truncate();
        }
        this.eventFolderState = undefined;
    }
    /**
     * Add geometry (detector geometry) folder to the menu.
     */
    addGeometryFolder() {
        // Phoenix menu
        if (this.eventFolderState === undefined) {
            this.geomFolder = this.phoenixMenuRoot.addChild('Detector', (value) => {
                this.sceneManager.groupVisibility(SceneManager.GEOMETRIES_ID, value);
            }, 'perspective');
        }
        this.geomFolder
            .addConfig({
            type: 'checkbox',
            label: 'Wireframe',
            isChecked: false,
            onChange: (value) => {
                this.sceneManager.wireframeGeometries(value);
            },
        })
            .addConfig({
            type: 'slider',
            label: 'Opacity',
            min: 0,
            max: 1,
            step: 0.01,
            allowCustomValue: true,
            onChange: (value) => {
                this.sceneManager.setGeometryOpacity(this.sceneManager.getObjectByName(SceneManager.GEOMETRIES_ID), value);
            },
        })
            .addConfig({
            type: 'slider',
            label: 'Scale',
            min: 0,
            max: 20,
            step: 0.01,
            allowCustomValue: true,
            onChange: (scale) => {
                this.sceneManager.scaleObject(this.sceneManager.getObjectByName(SceneManager.GEOMETRIES_ID), scale);
            },
        });
    }
    /**
     * Add geometry to the menu's geometry folder and set up its configurable options.
     * @param object Object to add to the UI menu.
     * @param menuSubfolder Subfolder in the menu to add the geometry to. Example `Folder > Subfolder`.
     */
    addGeometry(object, menuSubfolder) {
        const { name, material, visible } = object;
        const color = material?.color;
        let parentNode = this.geomFolder;
        if (menuSubfolder) {
            parentNode = this.geomFolder.findInTreeOrCreate(menuSubfolder);
        }
        // find out where the actual object name starts, providing that the name
        // is hierarchical and contents higher level menu names too
        let nameStart = name.lastIndexOf(' > ');
        if (nameStart < 0) {
            nameStart = 0; // case where there is no hierarchy
        }
        else {
            nameStart += 3; // skip the last ' > '
        }
        const objFolder = parentNode.addChild(name.substring(nameStart), (value) => {
            this.sceneManager.objectVisibility(object, value);
        });
        objFolder.toggleState = visible;
        objFolder
            .addConfig({
            type: 'color',
            label: 'Color',
            color: color ? `#${new Color(color).getHexString()}` : undefined,
            onChange: (value) => {
                this.sceneManager.changeObjectColor(object, value);
            },
        })
            .addConfig({
            type: 'slider',
            label: 'Opacity',
            min: 0,
            max: 1,
            step: 0.05,
            allowCustomValue: true,
            onChange: (opacity) => {
                this.sceneManager.setGeometryOpacity(object, opacity);
            },
        })
            .addConfig({
            type: 'button',
            label: 'Remove',
            onClick: () => {
                objFolder.remove();
                this.sceneManager.removeGeometry(object);
            },
        });
    }
    /**
     * Add event data folder with functions for event data toggles like show/hide and depthTest.
     */
    addEventDataFolder() {
        // Phoenix menu
        if (this.eventFolder) {
            this.eventFolderState = this.eventFolder.getNodeState();
            this.eventFolder.remove();
        }
        this.eventFolder = this.phoenixMenuRoot.addChild('Event Data', (value) => {
            this.sceneManager.groupVisibility(SceneManager.EVENT_DATA_ID, value);
        }, 'event-folder');
        this.eventFolder.addConfig({
            type: 'checkbox',
            label: 'Depth Test',
            isChecked: true,
            onChange: (value) => {
                this.three.eventDataDepthTest(value);
            },
        });
    }
    /**
     * Add folder for event data type like tracks or hits to the menu.
     * @param typeName Name of the type of event data.
     */
    addEventDataTypeFolder(typeName) {
        this.eventFolder.addChild(typeName, (value) => {
            this.sceneManager.objectVisibility(this.sceneManager
                .getObjectByName(SceneManager.EVENT_DATA_ID)
                .getObjectByName(typeName), value);
        });
    }
    /**
     * Add collection folder and its configurable options to the event data type (tracks, hits etc.) folder.
     * @param eventDataType Name of the event data type.
     * @param collectionName Name of the collection to be added in the type of event data (tracks, hits etc.).
     * @param cuts Cuts to the collection of event data that are to be made configurable to filter event data.
     * @param collectionColor Default color of the collection.
     */
    addCollection(eventDataType, collectionName, cuts, collectionColor) {
        const typeFolder = this.eventFolder.children.find((eventDataTypeNode) => eventDataTypeNode.name === eventDataType);
        if (!typeFolder) {
            return;
        }
        const collectionNode = typeFolder.addChild(collectionName, (value) => {
            const collectionObject = this.sceneManager
                .getObjectByName(SceneManager.EVENT_DATA_ID)
                .getObjectByName(collectionName);
            if (collectionObject)
                this.sceneManager.objectVisibility(collectionObject, value);
        });
        this.addDrawOptions(collectionNode, collectionName);
        if (cuts && cuts.length > 0) {
            this.addCutOptions(collectionNode, collectionName, cuts);
        }
        const colorByOptions = [];
        // Extra config options specific to tracks
        if (typeFolder.name === 'Tracks') {
            colorByOptions.push(ColorByOptionKeys.CHARGE, ColorByOptionKeys.MOM, ColorByOptionKeys.VERTEX);
        }
        new ColorOptions(this.three.getColorManager(), collectionNode, collectionColor ? collectionColor : new Color(), colorByOptions);
    }
    /**
     * Add Cut Options folder to the menu.
     * @param collectionNode The parent node to attach folder to
     * @param collectionName The name of the collection
     */
    addCutOptions(collectionNode, collectionName, cuts) {
        const cutsOptionsNode = collectionNode.addChild('Cut Options');
        cutsOptionsNode
            .addConfig({
            type: 'label',
            label: 'Cuts',
        })
            .addConfig({
            type: 'button',
            label: 'Reset cuts',
            onClick: () => {
                this.sceneManager.groupVisibility(collectionName, true, SceneManager.EVENT_DATA_ID);
                for (const cut of cuts) {
                    cut.reset();
                }
            },
        });
        // Add range sliders for cuts
        for (const cut of cuts) {
            cutsOptionsNode.addConfig(cut.getConfigRangeSlider(() => this.sceneManager.collectionFilter(collectionName, cuts)));
        }
    }
    /**
     * Add Draw Options folder to the menu.
     * @param collectionNode The parent node to attach folder to
     * @param collectionName The name of the collection
     */
    addDrawOptions(collectionNode, collectionName) {
        const drawOptionsNode = collectionNode.addChild('Draw Options');
        drawOptionsNode.addConfig({
            type: 'slider',
            label: 'Opacity',
            min: 0.1,
            step: 0.1,
            max: 1,
            onChange: (value) => {
                const eventDataObject = this.sceneManager.getObjectByName(SceneManager.EVENT_DATA_ID);
                if (eventDataObject) {
                    const collectionObject = eventDataObject.getObjectByName(collectionName);
                    if (collectionObject) {
                        this.sceneManager.setGeometryOpacity(collectionObject, value);
                    }
                }
            },
        });
        drawOptionsNode.addConfig({
            type: 'checkbox',
            label: 'Wireframe',
            onChange: (value) => this.sceneManager.wireframeObjects(this.sceneManager
                .getObjectByName(SceneManager.EVENT_DATA_ID)
                ?.getObjectByName(collectionName), value),
        });
    }
    /**
     * Add labels folder to the menu.
     * @param configFunctions Functions to attach to the labels folder configuration.
     */
    addLabelsFolder(configFunctions) {
        if (this.labelsFolder) {
            return;
        }
        const { onToggle, onSizeChange, onColorChange, onSaveLabels, onLoadLabels, } = configFunctions;
        this.labelsFolder = this.phoenixMenuRoot.addChild(SceneManager.LABELS_ID, onToggle, 'info');
        this.labelsFolder.addConfig({
            type: 'slider',
            label: 'Size',
            min: 0,
            max: 10,
            step: 0.01,
            allowCustomValue: true,
            onChange: onSizeChange,
        });
        this.labelsFolder.addConfig({
            type: 'color',
            label: 'Color',
            color: '#a8a8a8',
            onChange: onColorChange,
        });
        this.labelsFolder.addConfig({
            type: 'button',
            label: 'Save Labels',
            onClick: onSaveLabels,
        });
        this.labelsFolder.addConfig({
            type: 'button',
            label: 'Load Labels',
            onClick: onLoadLabels,
        });
    }
    /**
     * Add folder for configuration of label.
     * @param labelId Unique ID of the label.
     * @param onRemoveLabel Function called when label is removed.
     */
    addLabel(labelId, onRemoveLabel) {
        let labelNode = this.labelsFolder.children.find((phoenixMenuNode) => phoenixMenuNode.name === labelId);
        if (labelNode) {
            return;
        }
        labelNode = this.labelsFolder.addChild(labelId, (value) => {
            const labelObject = this.sceneManager
                .getObjectByName(SceneManager.LABELS_ID)
                .getObjectByName(labelId);
            if (labelObject)
                this.sceneManager.objectVisibility(labelObject, value);
        });
        labelNode.addConfig({
            type: 'color',
            label: 'Color',
            color: '#a8a8a8',
            onChange: (value) => {
                this.sceneManager.changeObjectColor(this.sceneManager.getObjectByName(labelId), value);
            },
        });
        labelNode.addConfig({
            type: 'button',
            label: 'Remove',
            onClick: () => {
                onRemoveLabel?.();
                this.removeLabel(labelId, labelNode);
            },
        });
    }
    /**
     * Remove label from the menu and scene if it exists.
     * @param labelId A unique label ID string.
     * @param labelFolderReference Reference to the label folder.
     */
    removeLabel(labelId, labelNode) {
        if (!labelNode) {
            labelNode = this.labelsFolder?.children.find((singleLabelNode) => singleLabelNode.name === labelId);
        }
        labelNode?.remove();
    }
    /**
     * Get the folder of the event data type.
     * @param typeName Name of the event data type.
     * @returns Folder of the event data type.
     */
    getEventDataTypeFolder(typeName) {
        return this.eventFolder.children.find((eventDataTypeNode) => eventDataTypeNode.name === typeName);
    }
    /**
     * Load previous state of the event data folder in Phoenix menu if any.
     */
    loadEventFolderState() {
        if (this.eventFolderState) {
            this.eventFolder.loadStateFromJSON(this.eventFolderState);
        }
    }
}
//# sourceMappingURL=phoenix-menu-ui.js.map