import { JiveXMLLoader } from '../loaders/jivexml-loader';
import { PhoenixLoader } from '../loaders/phoenix-loader';
import { StateManager } from './state-manager';
import { readZipFile } from '../helpers/zip';
/**
 * Model for Phoenix URL options.
 */
export const phoenixURLOptions = {
    file: '',
    type: '',
    config: '',
    hideWidgets: false,
    embed: false,
};
/**
 * A manager for managing options given through URL.
 */
export class URLOptionsManager {
    /**
     * Constructor for the URL options manager.
     * @param eventDisplay The Phoenix event display.
     * @param configuration Configuration of the event display.
     */
    constructor(eventDisplay, configuration) {
        this.eventDisplay = eventDisplay;
        this.configuration = configuration;
        this.urlOptions = new URLSearchParams(window.location.href.substr(window.location.href.lastIndexOf('?')));
    }
    /**
     * Initialize and apply all URL options on page load.
     */
    applyOptions() {
        // Initialize event with data from URL if there is any
        this.applyEventOptions(this.configuration.defaultEventFile?.eventFile, this.configuration.defaultEventFile?.eventType);
        this.applyHideWidgetsOptions();
        this.applyEmbedOption();
    }
    /**
     * Initialize the event display with event data and configuration from URL.
     * (Only JiveXML and JSON)
     * @param defaultEventPath Default event path to fallback to if none in URL.
     * @param defaultEventType Default event type to fallback to if none in URL.
     */
    applyEventOptions(defaultEventPath = '', defaultEventType = '') {
        if (!('fetch' in window)) {
            return;
        }
        let file, type;
        if ((!this.urlOptions.get('file') && this.urlOptions.get('type')) ||
            (this.urlOptions.get('file') && !this.urlOptions.get('type'))) {
            console.log('WARNING - if you set one of type or file, then you need to set both!');
            console.log('WARNING - reverting to defaults!');
        }
        if (!this.urlOptions.get('file') || !this.urlOptions.get('type')) {
            file = defaultEventPath;
            type = defaultEventType;
        }
        else {
            file = this.urlOptions.get('file') ?? '';
            type = this.urlOptions.get('type')?.toLowerCase() ?? '';
            console.log('Default file(', defaultEventPath, ') was overridden by URL options to: ', file);
        }
        console.log('Try to load event file: ', file, 'of type', type);
        // Try to load config from URL
        const loadConfig = () => {
            if (this.urlOptions.get('config')) {
                this.eventDisplay.getLoadingManager().addLoadableItem('url_config');
                fetch(this.urlOptions.get('config') ?? '')
                    .then((res) => res.json())
                    .then((jsonState) => {
                    console.log('Applying configuration ', this.urlOptions.get('config'), '  from urlOptions');
                    const stateManager = new StateManager();
                    stateManager.loadStateFromJSON(jsonState);
                })
                    .finally(() => {
                    this.eventDisplay.getLoadingManager().itemLoaded('url_config');
                });
            }
        };
        const processEventFile = (fileURL) => {
            if (type === 'jivexml') {
                console.log('Opening JiveXML');
                return this.handleJiveXMLEvent(fileURL);
            }
            else if (type === 'zip') {
                console.log('Opening zip file');
                return this.handleZipFileEvents(fileURL);
            }
            else {
                return this.handleJSONEvent(fileURL);
            }
        };
        // Load event file from URL
        if (file && type) {
            this.eventDisplay.getLoadingManager().addLoadableItem('url_event');
            processEventFile(file)
                .catch((error) => {
                this.eventDisplay
                    .getInfoLogger()
                    .add('Could not find the file specified in URL.', 'Error');
                console.error('Could not find the file specified in URL.', error);
            })
                .finally(() => {
                // Load config from URL after loading the event
                loadConfig();
                this.eventDisplay.getLoadingManager().itemLoaded('url_event');
            });
        }
        else {
            loadConfig();
        }
    }
    /**
     * Handle JiveXML event from file URL.
     * @param fileURL URL to the XML file.
     * @returns An empty promise. ;(
     */
    async handleJiveXMLEvent(fileURL) {
        const fileData = await (await fetch(fileURL).then((response) => {
            if (response.status >= 400 && response.status < 600) {
                throw new Error('Bad response from server');
            }
            return response;
        })).text();
        if (!this.configuration.eventDataLoader) {
            this.configuration.eventDataLoader = new JiveXMLLoader();
        }
        // Parse the XML to extract events and their data
        const loader = this.configuration.eventDataLoader;
        loader.process(fileData);
        const eventData = loader.getEventData();
        this.eventDisplay.buildEventDataFromJSON(eventData);
    }
    /**
     * Handle JSON event from file URL.
     * @param fileURL URL to the JSON file.
     * @returns An empty promise. ;(
     */
    async handleJSONEvent(fileURL) {
        const fileData = await (await fetch(fileURL)).json();
        this.configuration.eventDataLoader = new PhoenixLoader();
        this.eventDisplay.parsePhoenixEvents(fileData);
    }
    /**
     * Handle zip containing event data files.
     * @param fileURL URL to the zip file.
     * @returns An empty promise. ;(
     */
    async handleZipFileEvents(fileURL) {
        const fileBuffer = await (await fetch(fileURL)).arrayBuffer();
        const allEventsObject = {};
        let filesWithData;
        // Using a try catch block to catch any errors in Promises
        try {
            filesWithData = await readZipFile(fileBuffer);
        }
        catch (error) {
            console.error('Error while reading zip', error);
            this.eventDisplay.getInfoLogger().add('Could not read zip file', 'Error');
            return;
        }
        // JSON event data
        Object.keys(filesWithData)
            .filter((fileName) => fileName.endsWith('.json'))
            .forEach((fileName) => {
            Object.assign(allEventsObject, JSON.parse(filesWithData[fileName]));
        });
        // JiveXML event data
        const jiveloader = this.configuration.eventDataLoader instanceof JiveXMLLoader
            ? this.configuration.eventDataLoader
            : new JiveXMLLoader();
        Object.keys(filesWithData)
            .filter((fileName) => {
            return fileName.endsWith('.xml') || fileName.startsWith('JiveXML');
        })
            .forEach((fileName) => {
            jiveloader.process(filesWithData[fileName]);
            const eventData = jiveloader.getEventData();
            Object.assign(allEventsObject, { [fileName]: eventData });
        });
        // For some reason the above doesn't pick up JiveXML_XXX_YYY.zip
        this.eventDisplay.parsePhoenixEvents(allEventsObject);
    }
    /**
     * Hide all overlay widgets if "hideWidgets" option from the URL is true.
     */
    applyHideWidgetsOptions() {
        const hideWidgetsOptions = {
            hideWidgets: [
                'mainLogo', // Main logo
                'uiMenu', // UI menu
                'experimentInfo', // Experiment info
                'phoenixMenu', // Phoenix menu
                'statsElement', // Stats at the bottom left
                'gui', // dat.GUI menu
            ],
        };
        this.hideIdsWithURLOption(hideWidgetsOptions);
    }
    /**
     * Hide all overlay widgets and enable embed menu if "hideWidgets" option from the URL is true.
     */
    applyEmbedOption() {
        if (this.urlOptions.get('embed') === 'true') {
            const hideWidgetsOptions = {
                embed: [
                    'mainLogo', // Main logo
                    'uiMenu', // UI menu
                    'experimentInfo', // Experiment info
                    'phoenixMenu', // Phoenix menu
                    'statsElement', // Stats at the bottom left
                    'gui', // dat.GUI menu
                ],
            };
            this.hideIdsWithURLOption(hideWidgetsOptions);
            document
                .getElementById('embedMenu')
                ?.style.setProperty('display', 'block');
        }
    }
    /**
     * Hide element with IDs based on a URL option.
     * @param urlOptionWithIds IDs to hide with keys as the URL option and its array value as IDs.
     */
    hideIdsWithURLOption(urlOptionWithIds) {
        Object.entries(urlOptionWithIds).forEach(([urlOption, idsToHide]) => {
            if (this.urlOptions.get(urlOption) === 'true') {
                idsToHide.forEach((singleId) => {
                    document
                        .getElementById(singleId)
                        ?.style.setProperty('display', 'none');
                });
            }
        });
    }
    /**
     * Get options from URL set through query parameters.
     * @returns URL options.
     */
    getURLOptions() {
        return this.urlOptions;
    }
}
//# sourceMappingURL=url-options-manager.js.map